var Level = {
    
    game_running: false,
    
    collision_objects: [],
    
    boxes: [],
    
    current_level: 1,
    
    planet_objects: [],
    
    level_container: new PIXI.Container(),
    
    star_field_container: new PIXI.Container(),
    
    collision_explosion: null,
    
    scroll_position_x: 0,
    scroll_position_y: 0,
    scroll_area: 400,
    
    min_box_height: 100,
    max_box_height: 300,
    
    addPlanet: function(planet) {
        this.planet_objects.push(planet);
        this.level_container.addChild(planet.sprite);
        // We can collide with a planet
        this.addCollisionObject(planet);
    },
    
    addBox: function(x,y, points) {
        var box = new Box();
        box.sprite.x = x;
        box.sprite.y = y;
        box.points = points;
        this.boxes.push(box);
        this.level_container.addChild(box.sprite);
        this.addCollisionObject(box);
    },
    
    addCollisionObject: function(collision_object) {
        this.collision_objects.push(collision_object);
    },
    
    generateBox: function() {
        // Select a random planet
        var planet = this.planet_objects[Math.floor(Math.random()*this.planet_objects.length)];
        do {
            // Select a random direction and range
            var direction = Math.random()*Math.PI*2;
            var range = Math.random()*(this.max_box_height-this.min_box_height)+this.min_box_height+planet.diameter/2;
            // Calculate position
            var x = planet.sprite.x + Math.cos(direction)*range;
            var y = planet.sprite.y + Math.sin(direction)*range;
            // Check for collision with anything
            var collision = false;
            var best_points = 100;
            this.collision_objects.every(function(element) {
                var dist = distance(x,y, element.sprite.x, element.sprite.y);
                if (dist < element.diameter/2+20+this.min_box_height) {
                    collision = true;
                    return false;
                };
                if (element.type == 'planet') {
                    // Less than 50 px to surface gives 500 points
                    if (dist-element.diameter/2 < 50 && best_points < 500) best_points = 500;
                    // Less than 100 px to surface gives 400 points
                    if (dist-element.diameter/2 < 100 && best_points < 400) best_points = 400;
                    // Less than 150 px to surface gives 300 points
                    if (dist-element.diameter/2 < 150 && best_points < 300) best_points = 300;
                    // Less than 200 px to surface gives 200 points
                    if (dist-element.diameter/2 < 200 && best_points < 200) best_points = 200;
                    
                }
                return true;
            }.bind(this));
            if (collision) {
                continue;
            }
            break;
        } while(true);
        this.addBox(x,y, best_points);
    },
    
    star_field_texture: PIXI.Texture.from('gfx/star_field.png'),
    
    star_field_1: null,
    star_field_2: null,
    
    prepareStarfields: function() {
        var prev_state = this.game_running;
        this.game_running = false;
        this.star_field_container.removeChildren();
        this.star_field_1 = new PIXI.TilingSprite(this.star_field_texture, $(window).innerWidth(), $(window).innerHeight());
        this.star_field_2 = new PIXI.TilingSprite(this.star_field_texture, $(window).innerWidth(), $(window).innerHeight());
        this.star_field_container.addChild(this.star_field_1);
        this.star_field_container.addChild(this.star_field_2);
        this.star_field_2.tileScale.x = 0.7;
        this.star_field_2.tileScale.y = 0.7;
        this.game_running = prev_state;
    },
    
    start: function() {
        // Reset score
        Score.reset();
        Countdown.finished_callback = this.timeover;
        
        this.currentlevel = 1;
        this.set(this.currentlevel);
    },
    
    set: function(level) {
        // Clear stage
        game.stage.removeChildren();
        
        // Clear level
        this.level_container.removeChildren();
        this.level_container.setTransform(0,0);

        // Prepare starfields
        this.prepareStarfields();
        game.stage.addChild(this.star_field_container);

        // Place spaceship
        Spaceship.reset(1000,1000);
        Spaceship.sprite.visible = true;
        this.level_container.addChild(Spaceship.sprite);
        this.center_on_spaceship();
        
        // Clear objects
        this.collision_objects = [];
        this.planet_objects = [];
        
        // Clear boxes
        this.boxes = [];
        
        // Ready collision explosion
        this.collision_explosion = new Explosion();
        this.level_container.addChild(this.collision_explosion.sprite);
        
        switch(level) {
            case 1:
                // Planets on level 1
                var planet = new Planet(15, 500, 500, 2, 25);
                this.addPlanet(planet);
                var planet = new Planet(6, 900, 400, 0.5);
                this.addPlanet(planet);
                this.min_box_height = 100;
                this.max_box_height = 500;
                for (var i = 0; i < 6; i++) this.generateBox();
                Countdown.reset(30);
                break;
            case 2:
                // Planets on level 2
                var planet = new Planet(1, 0, 500);
                this.addPlanet(planet);
                var planet = new Planet(5, 1500, 500);
                this.addPlanet(planet);
                var planet = new Planet(12, 200, 200, 0.5);
                this.addPlanet(planet);
                var planet = new Planet(20, 600, 50, 0.5);
                this.addPlanet(planet);
                var planet = new Planet(3, 1000, 300, 0.5);
                this.addPlanet(planet);
                var planet = new Planet(16, 600, 1000);
                this.addPlanet(planet);
                this.min_box_height = 50;
                this.max_box_height = 400;
                for (var i = 0; i < 8; i++) this.generateBox();
                Countdown.reset(60);
                break;
            case 3:
                // Planets on level 3
                var planet = new Planet(6, 700, 700);
                this.addPlanet(planet);
                var planet = new Planet(4, 200, 200,2);
                this.addPlanet(planet);
                var planet = new Planet(5, 1400, 400);
                this.addPlanet(planet);
                var planet = new Planet(7, 2000, 560);
                this.addPlanet(planet);
                this.min_box_height = 50;
                this.max_box_height = 400;
                for (var i = 0; i < 3; i++) this.generateBox();
                Countdown.reset(60);
                break;
        }
        
        game.stage.addChild(this.level_container);
        
        // Add score
        Score.score_text.x = $(window).innerWidth()-250;
        Score.score_text.y = 20;
        game.stage.addChild(Score.score_text);
        
        // Add bigtext
        game.stage.addChild(BigCenter.text_object);
        BigCenter.center();
        BigCenter.fadeOut('LEVEL '+level);
        
        // Add counter
        Countdown.label.x = 20;
        Countdown.label.y = 20;
        game.stage.addChild(Countdown.label);
        
        this.game_running = true;
    },
    
    simulate: function(delta) {
        if (! this.game_running) return;
        this.collision_detector();
        this.gravity_logic(delta);
        this.level_scroller(delta);
        this.box_pulsate(delta);
    },
    
    gravity_logic: function(delta) {
        this.planet_objects.forEach(function(planet) {
            planet.getGravityPull(Spaceship.sprite.x, Spaceship.sprite.y);
        }.bind(this));
    },
    
    center_on_spaceship: function() {
        var global_position = Spaceship.sprite.getGlobalPosition();
        this.scroll_position_x = global_position.x-$(window).innerWidth()/2;
        this.scroll_position_y = global_position.y-$(window).innerHeight()/2;
        // Move level
        this.level_container.setTransform(-this.scroll_position_x, -this.scroll_position_y);
    },
    
    level_scroller: function(delta) {
        var global_position = Spaceship.sprite.getGlobalPosition();
        
        if (global_position.x < this.scroll_area) {
            // We need to scroll left
            this.scroll_position_x -= this.scroll_area-global_position.x;
        } else if (global_position.x > $(window).innerWidth()-this.scroll_area) {
            // We need to scroll right
            this.scroll_position_x += this.scroll_area-($(window).innerWidth()-global_position.x);
        }
        
        if (global_position.y < this.scroll_area) {
            // We need to scroll left
            this.scroll_position_y -= this.scroll_area-global_position.y;
        } else if (global_position.y > $(window).innerHeight()-this.scroll_area) {
            // We need to scroll right
            this.scroll_position_y += this.scroll_area-($(window).innerHeight()-global_position.y);
        }
        
        // Move level
        this.level_container.setTransform(-this.scroll_position_x, -this.scroll_position_y);
        
        this.star_field_1.tilePosition.x = -this.scroll_position_x/1.5;
        this.star_field_1.tilePosition.y = -this.scroll_position_y/1.5;
        this.star_field_2.tilePosition.x = -this.scroll_position_x/2;
        this.star_field_2.tilePosition.y = -this.scroll_position_y/2;        
    },
    
    box_pulsate: function(delta) {
        this.boxes.forEach(function(box) {
            box.pulsate(delta);
        });
    },
    
    box_pickup: function(box) {
        // Find box in array
        var i = this.boxes.indexOf(box);
        if (i > -1) this.boxes.splice(i,1);
        var i = this.collision_objects.indexOf(box);
        if (i > -1) this.collision_objects.splice(i,1);
        // Sound it
        PIXI.sound.play('pickup');
        // Score it
        Score.addScore(box.points);
        box.sprite.destroy();
        // Generate a new box
        Level.generateBox();
    },
        
    
    collision_detector: function() {
        this.collision_objects.forEach(function(v) {
            var dist = distance(Spaceship.sprite.x, Spaceship.sprite.y, v.sprite.x, v.sprite.y);
            if (dist < v.diameter/2+25) {
                // We collided. Check what we collided with?
                if (v.type == 'box') {
                    this.box_pickup(v);
                    return;
                }
                this.gameover();
                return;
            }
        }.bind(this));
    },
    
    timeover: function() {
        Level.game_running = false;
        Spaceship.sprite.visible = false;
        Level.currentlevel++;
        Level.set(Level.currentlevel);
    },
    
    gameover: function() {
        Countdown.stop();
        PIXI.sound.play('explosion');
        Level.game_running = false;
        Spaceship.sprite.visible = false;
        this.collision_explosion.explode(Spaceship.sprite.x, Spaceship.sprite.y);
        BigCenter.fadeIn('CRASHED!');
    }
    
}